# coding: utf-8
import sys
from PyQt5 import QtWidgets as qtw
from PyQt5 import QtCore as qtc
from PyQt5 import QtGui as qtg
from PyQt5.QtWidgets import *
from PyQt5.QtCore import QTimer, QDateTime
from monitorUI import Ui_MainWindow
from pressurem import Ui_PressureM
import time
import numpy as np
import serial
import qdarkstyle 
import serial.tools.list_ports
import plotMonitor


import ctypes
myappid = 'mycompany.myproduct.subproduct.version' # arbitrary string
ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
counter = 1

##global variable used to indicate plot Monitor has been activated
flag_PlotMonitorInit = False



#Pressure Monitor UI
class PressureM(qtw.QMainWindow):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.ui = Ui_PressureM()
        self.ui.setupUi(self)
        self.setWindowTitle("Pressure Monitor")
        self.setWindowIcon(qtg.QIcon('iconCat.jpg'))
    
        #signal-slot connection
        self.ui.pushButton_C0.released.connect(self.GeneratePlotMonitor)
        self.plotMonitorCounter = 0
        
        
    # def closeEvent(self, event):
        # if self.plotMonitor:
            # self.plotMonitor.close()    
    #update 
    def GeneratePlotMonitor(self):
        if (self.sender()==self.ui.pushButton_C0):
            plotname = 'Channel No.0'
        self.plotMonitor = plotMonitor.MainWindow(PlotName = plotname)
        self.plotMonitor.show()
        
        #self.plotMonitor.updateOnce(currentPressure,targetPressure,flowRateInflation,flowRateDeflation)
        
        global flag_PlotMonitorInit
        flag_PlotMonitorInit = True
        # if self.plotMonitorCounter%2==1:
            # print("Close plot monitor")
            # self.plotMonitor.close()
        # self.plotMonitorCounter = self.plotMonitorCounter + 1    
    def openPlotMonitor2(self):
        pass
    
#when want to show info in the left textedit, use 
#self.ui.textEdit_info.append(str())
#Control Panel UI

class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        
        self.setWindowTitle("OpenPneu")
        self.setWindowIcon(qtg.QIcon('iconCat.jpg'))
        
        self.ui.pushButton_4.setEnabled(False)
        self.ui.pushButton_5.setEnabled(False)
        
        self.ui.pushButton_connect.released.connect(self.Connect)
        self.ui.pushButton_disconnect.released.connect(self.Disconnect)
        self.ui.pushButton_Send.released.connect(self.SendCmd)
        
        self.ui.pushButton_StartRecording.released.connect(self.StartRecordData)
        self.ui.pushButton_PauseRecording.released.connect(self.PauseRecordData)
        self.ui.pushButton_SaveRecording.released.connect(self.SaveRecordData)
        
        self.recordCounter = 0
        
    def closeEvent(self, event):
        try:
            if self.w.plotMonitor:
                self.w.plotMonitor.close()
            if self.w:
                self.w.close()
        except:
            pass

    def SendCmd(self):
        self.command = self.ui.lineEdit_CMD.text()
        try:
            if self.serial.is_open:
                #self.serial.reset_input_buffer()
                self.serial.write(bytes(self.command+"\r\n", 'utf-8'))
                #answer = self.serial.read_until(expected=b"\n")
                #print("before is: ",answer)
                #answer = answer.decode("utf-8")
                #self.ShowInfo(answer)
                #print("Info is: ",answer)
        except:
            print("Except happens, plz check your connection with board...")
            
        
    #If want to show info in the left textedit, use self.ShowInfo(string type)
    def ShowInfo(self, info):
        self.ui.textEdit_info.append(info)
        
    def Connect(self):
        self.ui.pushButton_disconnect.setEnabled(True)
        self.serial = serial.Serial()
        time.sleep(1)
        self.serial.baudrate = 115200
        self.serial.timeout = 0.3
        self.plist = list(serial.tools.list_ports.comports())
        print(self.plist[0])
        #self.ShowInfo(self.plist)
        flag_success = False
        if len(self.plist) <= 0: # 无串口
            self.ShowInfo("no port")
        else:
            # plist_0 = list(self.plist[0])
            # self.ShowInfo(plist_0[0])
            
            #traverse all serial ports and find the right port
            for port in self.plist:
                plist_0 = list(port)
                portName = plist_0[0]
                print(portName)
                self.serial.port = portName
                self.serial.open()
                time.sleep(2.5) #this delay is waiting for CPU to going into main loop
                print("After Initialization")
                self.serial.reset_input_buffer()
                self.serial.write(bytes("AT+ADDRESS\r\n", 'utf-8'))
                answer = self.serial.read_until(expected=b"\n")
                print("before is: ",answer)
                answer = answer.decode("utf-8")
                #self.ShowInfo(answer)
                print("Info is: ",answer)
                if answer == "OpenPneu2.0\r\n":
                    self.ShowInfo("Connect to:\nOpenPneu2.0\nRight port, keep connection")
                    
                    flag_success = True
                    
                    self.ui.pushButton_5.setEnabled(True)
                    
                    break
                else:
                    self.serial.reset_input_buffer() 
                    self.ShowInfo("Error port, switch to next...")
                
            if flag_success == True:
                self.ShowInfo("Connection established...\n")
                self.ui.pushButton_connect.setEnabled(False)
            else:
                self.ShowInfo("No correct port has been found...\n")
             
    def Disconnect(self):
        self.serial.close()
        self.ui.pushButton_connect.setEnabled(True)
        self.ui.pushButton_disconnect.setEnabled(False)
        self.ShowInfo("Port closed..\n")
        
    def UpdateClock(self):
        pass

    def write_read(self,x):
        self.arduino = serial.Serial(port='/dev/cu.usbmodem1201', baudrate=115200, timeout=.1)
        self.arduino.write(bytes(x, 'utf-8'))
        time.sleep(0.05)
        self.data = self.arduino.readline()
        return self.data

    def OnRelease_Run(self):
        #readData = np.loadtxt("testPressureData.txt")
        #timer definition
        self.timer = QTimer()
        self.timer.setInterval(5)
        self.timer.timeout.connect(self.lcdUpdate)
        self.counterTimer = 0
        self.timer.start()
        
        self.ui.pushButton_3.setEnabled(True)
        self.ui.pushButton_4.setEnabled(False)
     
    #After click on RUN, you will run this function
    def lcdUpdate(self):
        if self.w.isVisible():
            pass
        else:
            self.OnRelease_Stop()
            self.ShowInfo("Pressur monitor already closed, timer will stop...")
            return
    
        if self.serial.is_open is False:
            self.ShowInfo("Port not open, try later..\n")
            return
        try:
            self.serial.reset_input_buffer()
            self.serial.write(b"AT+UPDATE\r\n")
            answer = self.serial.read_until().decode("utf-8")
            #self.ShowInfo(answer)
            flag_success = False
            if answer[-2:] == "\r\n":
                self.ShowInfo("Read data already...")
                flag_success = True
                
            else:
                self.serial.reset_input_buffer() 
                self.ShowInfo("Error message...")
            
            answer = answer[:-2]
            self.ShowInfo(answer)
            #15.68,5.00,0,0.00,0.00
            end1 = answer.find(',',0)
            #                       >=0, <end1
            currentP = float(answer[0:end1])
            end2 = answer.find(',',end1+1)
            targetP = float(answer[end1+1:end2])
            
            end3 = answer.find(',',end2+1)
            valvePos = int(answer[end2+1:end3])
            end4 = answer.find(',',end3+1)
            pumpSpeed1 = float(answer[end3+1:end4])
            pumpSpeed2 = float(answer[end4+1:])
      
            self.counterTimer = self.counterTimer + 1
            if self.counterTimer >= 120:
                self.counterTimer = 0

            self.w.ui.widget.updateValue(currentP)
        
            #update plot monitor values
            global flag_PlotMonitorInit
            if flag_PlotMonitorInit == True:
                self.w.plotMonitor.updateOnce(currentP,targetP,pumpSpeed1,pumpSpeed2)
                pass
                #print("RUNNN")
            else:
                pass
                #print("NOT RUN")
        except:
            pass
            
    def OnRelease_Stop(self):
        self.timer.stop()
        self.ui.pushButton_3.setEnabled(False)
        self.ui.pushButton_4.setEnabled(True)
        
        

    def OnRelease_Low(self):
        pass

    def OnRelease_High(self):
        pass

    def OnRelease_Connect(self):
        pass

    def OnRelease_Disconnect(self):
        pass

    #open the dial plate
    def OnRelease_Open(self):
        self.w = PressureM()
        self.w.show()
        self.ui.pushButton_4.setEnabled(True)

    def OnRelease_Close(self):
        self.w.close()
    
    #record the data
    def StartRecordData(self):
        if self.recordCounter == 0:
            self.recordTimer = QTimer()
            self.recordTimer.setInterval(1)
            self.recordTimer.timeout.connect(self.TimerFunc_Record)
            self.recordTimer.start()
            self.ShowInfo("Timer Initialized, will keep receiving the feedback information\n")
        else:
            self.recordTimer.start()
            
        self.recordCounter = self.recordCounter + 1
       
    #should be ran in other thread
    def TimerFunc_Record(self):
        # while True:
        try:
            if self.serial.in_waiting>=1:  
                answer = self.serial.read_until(expected=b"\n").decode("utf-8")
                if len(answer)<=1:
                    print("Less than 1 byte")
                    return
                if answer[-2:] == "\r\n":
                    if answer[0] == "T":
                        answer = answer[:-2]
                        #self.ShowInfo(answer)
                        #15.68,5.00,0,0.00,0.00
                        end1 = answer.find(',',0)
                        #                       >=0, <end1
                        currentP = float(answer[1:end1])
                        end2 = answer.find(',',end1+1)
                        targetP = float(answer[end1+1:end2])
                        
                        end3 = answer.find(',',end2+1)
                        valvePos = int(answer[end2+1:end3])
                        end4 = answer.find(',',end3+1)
                        pumpSpeed1 = float(answer[end3+1:end4])
                        pumpSpeed2 = float(answer[end4+1:])
                        print(f"Decode: {currentP},{targetP},{pumpSpeed1},{pumpSpeed2}")
        except:
            pass
        #print("In updating phase")
        #self.recordTimer.stop()
        
    def PauseRecordData(self):
        self.ShowInfo("PauseRecord\n")
        self.recordTimer.stop()
        pass
    def SaveRecordData(self):
        self.ShowInfo("SaveRecord\n")
        pass



if __name__ == '__main__':
    app = QApplication([])
    app.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())   #dark style
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())